# $Id: reblog.pl,v 1.11.2.3 2005/09/22 23:01:14 mfrumin Exp $
#
# This file is part of Reblog: http://reblog.org
# Distributed under the Gnu Public License, see LICENSE
#
# Copyright 2004 Michael Frumin, Michal Migurski
# This is the Movable Type Plugin code to implement the reBlog functionality into Movable Type
#
# Developed by Michael Frumin (mfrumin AT eyebeam DOT org)
# Initially for Eyebeam's reBlog (http://www.eyebeam.org/reblog/)
#
# NOTE: This only works if you are using MySQL for the database backend for Movable Type

use strict;
use MT;
use MT::Plugin;
use MT::PluginData;
use MT::Template::Context;
use MT::Object;

############################## PACKAGE FOR MT::ReblogData OBJECT #########################

package MT::ReblogData;
use strict;

use MT::Object;
@MT::ReblogData::ISA = qw( MT::Object );
__PACKAGE__->install_properties({
# FOR MT >= 3.2
    column_defs => {
	'id' => 'integer not null auto_increment',
	'entry_id' => 'integer not null',
	'link' => 'string(255) not null',
	'guid' => 'string(255) not null',
	'source_author' => 'string(255)',
	'via_link' => 'string(255) not null',
	'orig_created_on' => 'datetime not null',
	'source' => 'string(255) not null',
	'source_url' => 'string(255) not null',
	'source_feed_url' => 'string(255) not null'
	}
    ,
    
# FOR MT <= 3.1x
    columns => [ qw( id entry_id link guid source_author via_link orig_created_on source source_url source_feed_url) ],
    
    indexes => {
	post_hash => 1,
	entry_id => 1,
	created_on => 1,
    },
    audit => 1,
    datasource => 'reblog_data',
    primary_key => 'id',
});

########################## END PACKAGE FOR MT::ReblogData OBJECT #########################


############################## ADD THE PLUGIN LINK INTO MT PAGES ########################
package MT::Plugin::Reblog;
use strict;


$MT::Plugin::Reblog::VERSION = 1.1;

our $Instance;

eval {

$Instance = new MT::Plugin({name => "reBlog",
			    description => "Plugin for automatic syndication with attribution.",
			    config_link => "reblog_action.cgi",
			    doc_link => "http://www.reblog.org/"});
MT->add_plugin($Instance);

add_plugin_actions();

};
if($@) {
    print STDERR "ERROR: $@: $!\n";
}

sub add_plugin_actions {
    my $mt_vers = MT->version_number;
    my $blog_id = get_curr_blog_id();

    my ($status, $version) = MT::Plugin::Reblog::check_reblog_status();

    my (@actions);

    my ($text, $link);

    if($status eq '-mysql') {
	push(@actions, ["", "Sorry, but you can only use the reBlog plugin if you are using MySQL"]);
	$text = "Sorry, but you can only use the reBlog plugin if you are using MySQL";
	$link = "";
    }
    elsif($status eq '-install') {
	push(@actions, ["reblog_install.cgi", "Install the reBlog plugin"]);
	$text = "Install the reBlog plugin";
	$link = "reblog_install.cgi";
    }
    elsif($status eq '-upgrade') {
	push(@actions, ["reblog_upgrade.cgi", "Upgrade the reBlog plugin"]);
	$text = "Upgrade the reBlog plugin";
	$link = "reblog_upgrade.cgi";
    }
    else {

	my $source_rss = source_rss($blog_id);
	
	push(@actions, ["reblog_action.cgi?action=config" , "Configure your Reblog Source Feed"]);

	if($source_rss) {
	    push(@actions, ["reblog_action.cgi?action=import", "Import Entries from your Reblog Source Feed"]);
	}
    }

    if(defined($text) && $text) {
	$Instance->name($text);
	$Instance->description($text);	
	$Instance->config_link($link);	
    }

    foreach my $a (@actions) {
#	print STDERR "ACTION: $a[0] $a[1]\n";
	my (@la) = @$a;
	unshift(@la, "list_entries");
	if($mt_vers < 3.1) {
	    $la[1] .= "&blog_id=$blog_id";
	}

	MT->add_plugin_action(@la);
	
	if($mt_vers >= 3.1) {
	    my (@ba) = @$a;
	    unshift(@ba, "blog");
	    MT->add_plugin_action(@ba);
	}
    }
}

sub source_rss {
    my $blog_id = shift;
    my $source_rss = shift;

    if(defined($blog_id) && $blog_id) {
	my $pref = config_key_prefix($blog_id);
	my $data = MT::PluginData->load({ plugin => $Instance->name,
					  key    => $pref . "source_rss" });

	if(defined($source_rss) && $source_rss) {
	    if(!$data) {
		$data = MT::PluginData->new;
		$data->plugin($Instance->name);
		$data->key($pref . "source_rss");
	    }
	    $data->data(\$source_rss);
	    $data->save;
	}
	if($data) {
	    return ${$data->data};
	}
	else {
	    return '';
	}
    }

    return '';
}


sub config_key_prefix {
    my ($blog_id) = shift;

    return $blog_id . "-";
}

sub get_curr_blog_id {
    $_ = $ENV{REQUEST_URI};
    my ($blog_id) = /blog\_id\=(\d+)\b/i;
    if(!defined($blog_id)) {
	$blog_id = 0;
    }
    return $blog_id;
}

sub import_entries {
    my ($blog_id, $source_rss, $author) = @_;

    use LWP::Simple;
    use XML::XPath;
    use XML::XPath::XMLParser;
    use MT::Blog;
    use MT::Author;
    use HTTP::Date;
    use MT::Util qw( format_ts offset_time_list offset_time);
    use POSIX;

    my $mt_vers = MT->version_number;

    my (@entries);

    my $blog = MT::Blog->load($blog_id);

    if(!ref($author)) {
	$author = MT::Author->load($author);
    }
    
    my $rss = LWP::Simple::get($source_rss);

    if($rss =~ /^\<\?xml[^>]*encoding\=\"US\-ASCII/is) {
# clean up bogus characters
	my $fix = "[^\x09-\x0A\x0D\x20-\x7F]";
	$rss =~ s/$fix/ /igs;
    }

    if(defined($rss) && $rss) {
	my $xp = XML::XPath->new(xml => $rss);
	my $nodeset = $xp->findnodes("//item");
	
	while(my $node = $nodeset->shift()) {
	    my $body = $xp->findvalue("description", $node);
	    my $title = $xp->findvalue("title", $node);
	    my $link = $xp->findvalue("link", $node);
	    my $date = $xp->findvalue("dc:date", $node); 
	    my $subjects = $xp->findvalue("dc:subject", $node);
	    my $guid = $xp->findvalue("rb:guid", $node); 
	    my $via_link = $xp->findvalue("rb:via_url", $node);
	    my $orig_date = $xp->findvalue("rb:source_published_date", $node);
	    my $source_author = $xp->findvalue("rb:source_author", $node);
	    my $source_name = $xp->findvalue("rb:source", $node);
	    my $source_url = $xp->findvalue("rb:source_url", $node);
	    my $source_feed_url = $xp->findvalue("rb:source_feed_url", $node);

	    $date = HTTP::Date::parse_date($date);
	    my $ts = HTTP::Date::str2time($date);
	    $date = POSIX::strftime(($mt_vers < 3.2 ? "%Y-%m-%d %H:%M:%S" : "%Y%m%d%H%M%S"),
				    offset_time_list($ts, $blog));

	    $orig_date = HTTP::Date::parse_date($orig_date);
	    my $orig_ts = HTTP::Date::str2time($orig_date);
 	    $orig_date = POSIX::strftime(($mt_vers < 3.2 ? "%Y-%m-%d %H:%M:%S" : "%Y%m%d%H%M%S"),
					 offset_time_list($orig_ts, $blog));

# 	    $orig_date = POSIX::strftime("%Y-%m-%d %H:%M:%S",
#					 offset_time_list($orig_ts, $blog));

	    #can't use MT::Object join for this circumstance, so do it manually
	    my (@rb_data) = MT::ReblogData->load({ guid => $guid },
						 {sort => 'created_on', direction => 'ascend'});
	    my $rb_data = 0;
	    foreach my $rbd (@rb_data) {
		my $entry = MT::Entry->load({id => $rbd->entry_id, blog_id => $blog_id});
		if($entry) {
		    $rb_data = $rbd;
		}
	    }
	    
	    if(!$rb_data) {
		$rb_data = MT::ReblogData->new;
		$rb_data->created_on($date);
	    };

	    $rb_data->link($link);
	    $rb_data->guid($guid);
	    $rb_data->via_link($via_link);
	    $rb_data->orig_created_on($orig_date);
	    $rb_data->source_author($source_author);
	    $rb_data->source($source_name);
	    $rb_data->source_url($source_url);
	    $rb_data->source_feed_url($source_feed_url);

	    my $entry;
	    if(!($rb_data->entry_id && ($entry = MT::Entry->load({id => $rb_data->entry_id, blog_id => $blog_id})))) {
		$entry = MT::Entry->new();
		$entry->blog_id($blog_id);
		$entry->status(MT::Entry::RELEASE);
		$entry->allow_comments($blog->allow_comments_default);
		$entry->allow_pings($blog->allow_pings_default);
		$entry->convert_breaks(0);
		$entry->author_id($author->id);
		$entry->keywords($subjects);
		$entry->created_on($date);
	    }


	    my $ts_modified = HTTP::Date::str2time(format_ts("%Y-%m-%d %H:%M:%S", $entry->modified_on));

	    my $status = 'old';

	    my $categories = [];
	    if((!$entry->id) || 
	       ($ts > $ts_modified)) {
		$status = ($entry->id ? 'update' : 'new');

		$entry->title($title);
		$entry->text($body);
		$entry->basename(MT::Util::make_unique_basename($entry));

		$entry->modified_on($date);
		$entry->save || die "ENTRY SAVE FAILURE: " . $entry->errstr;
		my $eid = $entry->id;
		$entry = MT::Entry->load($entry->id );
		$categories = assign_categories($entry, $blog, $author, $subjects);


		$rb_data->modified_on($date);
		$rb_data->entry_id($entry->id);
		$rb_data->save || die "RBDATA SAVE FAILURE: " . $rb_data->errstr;
		$rb_data = MT::ReblogData->load($rb_data->id);

	    }

	    my $eRec = { entry => $entry, status => $status, reblog_data => $rb_data, categories => $categories };


	    push(@entries, $eRec);

	}
    }
    else {
	die "Failed to fetch RSS feed from $source_rss\n";
    }

    return (@entries);
    
}

sub assign_categories {
    my ($entry, $blog, $author, $subjects) = @_;

    my (@cats) = MT::Category->load({blog_id => $blog->id});

    my %cat_hash;
    foreach my $cat (@cats) {
	$cat_hash{lc($cat->label)} = $cat;
    }

    my (@placements) = MT::Placement->load({entry_id => $entry->id});
    my %place_hash;

    foreach my $place (@placements) {
	$place_hash{$place->category_id} = $place;
    }

    my (@subs) = split(/\s/, $subjects);
    my $res = [];
    my $primary = 0;
    foreach my $sub (@subs) {
	my (@cats) = split(/\:\:/, $sub);
	my $parent = 0;
	foreach my $cat (@cats) {
	    $cat =~ s/\+/ /igs;

	    if(exists($cat_hash{lc($cat)})) {
		$cat = $cat_hash{lc($cat)};
	    }
	    else {
		$cat = create_category($cat, $blog, $author, $parent);
		$cat_hash{lc($cat->label)} = $cat;
	    }

	    my $place;
	    if(exists($place_hash{$cat->id})) {
		$place = $place_hash{$cat->id};
	    }
	    else {
		$place = create_placement($entry, $cat, 0);
		$place_hash{$cat->id} = $place;
	    }
	    
	    if($place->is_primary) {
		$primary = $place;
	    }
	    $parent = $cat;
	    push(@$res, $cat);
	}
    }

    my %curr_cats;
    foreach my $cat (@$res) {
	$curr_cats{$cat->id} = $cat;
    }

    foreach my $place (values %place_hash) {
	if(!exists($curr_cats{$place->category_id})) {
	    $place->remove();
	}
    }
    
    if(!$primary && scalar(@$res) > 0) {
	$primary = $place_hash{$res->[0]->id};
	$primary->is_primary(1);
	$primary->save();
    }


    return $res;
}

sub create_category {
    my ($label, $blog, $author, $parent) = @_;

    my $cat = MT::Category->new();
    $cat->blog_id($blog->id);
    $cat->allow_pings($blog->allow_pings_default);
    $cat->label($label);
    $cat->author_id($author->id);
    $cat->parent(($parent ? $parent->id : 0));

    $cat->save();
    return MT::Category->load($cat->id);
}

sub create_placement {
    my ($entry, $cat, $is_primary) = @_;
    
    my $place = MT::Placement->new();
    $place->entry_id($entry->id);
    $place->blog_id($entry->blog_id);
    $place->category_id($cat->id);
    $place->is_primary($is_primary);
    $place->save();

    return MT::Placement->load($place->id);

}


sub check_reblog_status {

    my $driver = MT::Object->driver();

    if(ref($driver) !~ /mysql/i) {
	return '-mysql';
    }

    
    my $dbh = $driver->{dbh};
    
    my $ds = MT::ReblogData->datasource;


    my $q = $dbh->prepare("show tables");
    $q->execute();
    my %tables;
    while(my ($t) = $q->fetchrow_array) {
	$tables{$t} = 1;
    }
    $q->finish;
    
    if(!exists($tables{"mt_$ds"})) {
	my $e_ds = MT::Entry->datasource;

	my $cols = {};
	$q = $dbh->prepare("show columns from mt_$e_ds");
	$q->execute();
	while(my $c = $q->fetchrow_hashref) {
	    $cols->{$c->{Field}} = $c;
	}
	$q->finish;
	
	if(exists($cols->{$e_ds . '_reblog_source'})) {
	    return ('-upgrade', 0.1);
	}

	return '-install';
    }
    

    my $cols = {};
    $q = $dbh->prepare("show columns from mt_$ds");
    $q->execute();
    while(my $c = $q->fetchrow_hashref) {
	$cols->{$c->{Field}} = $c;
    }
    $q->finish;

    if(exists($cols->{$ds . '_post_hash'})) {
	return ('-upgrade', 0.9);
    }

    return 0;
    
}

sub create_reblog_data_table {
    my $driver = MT::Object->driver();
    my $dbh = $driver->{dbh};
    
    my $ds = MT::ReblogData->datasource;

    my $body = "CREATING $ds TABLE<BR>\n";
    
    my $sql = qq{create table mt_$ds (
				      ${ds}_id integer not null auto_increment primary key,
				      ${ds}_entry_id integer not null,
				      ${ds}_link varchar(255) not null,
				      ${ds}_guid varchar(255) not null,
				      ${ds}_source_author varchar(255),
				      ${ds}_via_link varchar(255) not null,
				      ${ds}_orig_created_on datetime,
				      ${ds}_source varchar(255) not null,
				      ${ds}_source_url varchar(255) not null,
				      ${ds}_source_feed_url varchar(255) not null,
				      ${ds}_created_on datetime not null,
				      ${ds}_modified_on timestamp not null,
				      ${ds}_created_by integer,
				      ${ds}_modified_by integer,
				      unique index (${ds}_entry_id),
				      index(${ds}_guid),
				      index (${ds}_created_on)
				      )
		 };
    
    if($dbh->do($sql)) {
	$body .= "TABLE CREATED<BR>\n";
    }
    else {
	$body .= "ERROR: " . $dbh->errstr . "<BR>\n";
    }

    return $body;

}


sub cleanup_reblog_data {
    my (@rbds) = MT::ReblogData->load();
    
    my $body = '';

    foreach my $rbd (@rbds) {
	$body .=  "Found MT::ReblogData id=" . $rbd->id . ", entry_id=" . $rbd->entry_id . "<BR>";
	my $e = MT::Entry->load({id => $rbd->entry_id});
	if($e) {
	    $body .= "Found an MT::Entry title=" . $e->title . "<BR>\n";
	}
	else {
	    $body .= "No corresponging MT::Entry.  Deleting this MT::ReblogData<BR>\n";
	    $rbd->remove();
	}
	$body .= "<BR>\n";
    }
    
    return $body;
}




############################## END ADD THE PLUGIN LINK INTO MT PAGES ####################





package main;
################################## TAG HANDLERS FOR REBLOG DATA ##########################

    MT::Template::Context->add_tag(EntryPrepareReblogData =>  \&_hdlr_entry_prepare_reblog_data);
    MT::Template::Context->add_tag(EntryReblogSource =>  \&_hdlr_entry_reblog_source);

    MT::Template::Context->add_tag(EntryReblogSourceLink =>  \&_hdlr_entry_reblog_source_url);
    MT::Template::Context->add_tag(EntryReblogSourceURL =>  \&_hdlr_entry_reblog_source_url);

    MT::Template::Context->add_tag(EntryReblogSourceLinkXML =>  \&_hdlr_entry_reblog_source_feed_url);
    MT::Template::Context->add_tag(EntryReblogSourceFeedURL =>  \&_hdlr_entry_reblog_source_feed_url);

    MT::Template::Context->add_tag(EntryReblogLink =>  \&_hdlr_entry_reblog_link);

    MT::Template::Context->add_tag(EntryReblogViaLink =>  \&_hdlr_entry_reblog_via_link);

    MT::Template::Context->add_tag(EntryReblogSourcePublishedDate =>  \&_hdlr_entry_reblog_orig_date);

    MT::Template::Context->add_tag(EntryReblogSourceAuthor =>  \&_hdlr_entry_reblog_source_author);
    MT::Template::Context->add_tag(EntryReblogAuthor =>  \&_hdlr_entry_reblog_source_author);

    MT::Template::Context->add_tag(EntryReblogIdentifier =>  \&_hdlr_entry_reblog_identifier);


    MT::Template::Context->add_conditional_tag(EntryIfReblog => sub {
	my $e = $_[0]->stash('entry')
	    or return $_[0]->_no_entry_error('MTEntryIfReblog');
	my $rbd = $_[0]->stash('reblog_data');
	return defined($rbd) && $rbd;
    });

    MT::Template::Context->add_conditional_tag(EntryIfNotReblog => sub {
	my $rbd = $_[0]->stash('reblog_data');
	return !(defined($rbd) && $rbd);
    });

    MT::Template::Context->add_conditional_tag(EntryIfHasReblogAuthor => sub {
	my $rbd = $_[0]->stash('reblog_data');
	return defined($rbd) && $rbd && $rbd->source_author;
    });


sub _hdlr_entry_prepare_reblog_data {
    my $e = $_[0]->stash('entry')
	or return $_[0]->_no_entry_error('MTEntryPepareReblogData');

    my $rb_data = MT::ReblogData->load({entry_id => $e->id});

    $_[0]->stash('reblog_data', $rb_data);
    '';
}
    

sub _hdlr_entry_reblog_source {
    my $rbd = $_[0]->stash('reblog_data')
        or return $_[0]->_no_entry_error('MTEntryReblogSource');
    defined($rbd) && $rbd ? $rbd->source : '';
}

sub _hdlr_entry_reblog_source_url {
    my $rbd = $_[0]->stash('reblog_data')
        or return $_[0]->_no_entry_error('MTEntryReblogSourceURL');
    defined($rbd) && $rbd ? $rbd->source_url : '';
}

sub _hdlr_entry_reblog_source_feed_url {
    my $rbd = $_[0]->stash('reblog_data')
        or return $_[0]->_no_entry_error('MTEntryReblogSourceFeedURL');
    defined($rbd) && $rbd ? $rbd->source_feed_url : '';
}

sub _hdlr_entry_reblog_link {
    my $rbd = $_[0]->stash('reblog_data')
        or return $_[0]->_no_entry_error('MTEntryReblogLink');
    defined($rbd) && $rbd ? $rbd->link : '';
}

sub _hdlr_entry_reblog_via_link {
    my $rbd = $_[0]->stash('reblog_data')
        or return $_[0]->_no_entry_error('MTEntryReblogViaLink');
    defined($rbd) && $rbd ? $rbd->via_link : '';
}

sub _hdlr_entry_reblog_orig_date {
    my $rbd = $_[0]->stash('reblog_data')
        or return $_[0]->_no_entry_error('MTEntryReblogSourcePublishedDate');

    my $att = $_[1];
    $att->{ts} = ($rbd ? MT::Object->driver()->db2ts($rbd->orig_created_on) : 0);

    defined($rbd) && $rbd ? 
	MT::Template::Context::_hdlr_date($_[0], $att)
	: '';
}

sub _hdlr_entry_reblog_source_author {
    my $rbd = $_[0]->stash('reblog_data')
        or return $_[0]->_no_entry_error('MTEntryReblogSourceAuthor');
    defined($rbd) && $rbd ? $rbd->source_author : '';
}

sub _hdlr_entry_reblog_identifier {
    my $rbd = $_[0]->stash('reblog_data')
        or return $_[0]->_no_entry_error('MTEntryReblogIdentifier');
    defined($rbd) && $rbd ? $rbd->guid : '';
}

############################## END TAG HANDLERS FOR REBLOG DATA ##########################






################################## TAG HANDLERS FOR REBLOG SOURCES #######################

MT::Template::Context->add_container_tag(ReblogSources => sub {
    my($ctx, $args, $cond) = @_;

    my $res = '';
    my $builder = $ctx->stash('builder');
    my $tokens = $ctx->stash('tokens');
    my $blog = $ctx->stash('blog'); 


#              range
#           To be used together with an array reference as the value for a col-
#           umn in \%terms; specifies that the specific column should be
#           searched for a range of values, rather than one specific value.

#           The value of range should be a hash reference, where the keys are
#           column names, and the values are all 1; each key specifies a column
#           that should be interpreted as a range.

    my $now = time;
    my $then;

    use MT::Util qw( format_ts offset_time_list offset_time);
    my $mt_vers = MT->version_number;

    if(exists($args->{max_item_age}) && $args->{max_item_age} =~ /^\d+$/) {
	$then = $now - $args->{max_item_age} * 24 * 60 * 60;
    }
    else {
	$then = 0;
    }

    
    $now = POSIX::strftime(($mt_vers < 3.2 ? "%Y-%m-%d %H:%M:%S" : "%Y%m%d%H%M%S"), offset_time_list($now, $blog));
    $then = POSIX::strftime(($mt_vers < 3.2 ? "%Y-%m-%d %H:%M:%S" : "%Y%m%d%H%M%S"), offset_time_list($then, $blog));	
    
    my @rbds = MT::ReblogData->load({created_on => [$then, $now]}, 
				    {sort => 'created_on', direction => 'descend'
					 , range => {created_on => 1}
			         }
				 );


    $now = POSIX::strftime("%Y%m%d%H%M%S", offset_time_list(time, $blog));
    
    my %sources;
    my $total_entry = 0;
    for my $rb_data (@rbds) {
	my $entry = MT::Entry->load( { id => $rb_data->entry_id, blog_id => $blog->id} );
	next if(!($entry && defined($rb_data->source_feed_url) && $rb_data->source_feed_url));
	if(!exists($sources{$rb_data->source_url})) {
	    $sources{$rb_data->source_url} = { n => 0, url => $rb_data->source_url, feed_url => $rb_data->source_feed_url, title => $rb_data->source, min_ts => $now, max_ts => POSIX::strftime("%Y%m%d%H%M%S", offset_time_list(0, $blog))};
	}
	my $source = $sources{$rb_data->source_url};

	if($rb_data->created_on le $source->{min_ts}) {
	    $source->{min_ts} = $rb_data->created_on;
	}

	if($rb_data->created_on ge $source->{max_ts}) {
	    $source->{max_ts} = $rb_data->created_on;
	}

	$source->{n}++;
	$total_entry++;
    }
    
    my (@sources) = sort {$b->{n} <=> $a->{n}} values %sources;

    if(exists($args->{max_feed_age}) && $args->{max_feed_age} =~ /^\d+$/) {
	$then = time - $args->{max_feed_age} * 24 * 60 * 60;
    }
    else {
	$then = 0;
    }

    $then = POSIX::strftime("%Y%m%d%H%M%S", offset_time_list($then, $blog));	

    foreach my $source (@sources) {
	$source->{nTotal} = $total_entry;

	if($source->{max_ts} ge $then) {
	    $ctx->stash('reblog_source', $source);
	    defined(my $out = $builder->build($ctx, $tokens))
		or return $ctx->error($ctx->errstr);
	    $res .= $out;
	}
    }
    
    $res;
});

MT::Template::Context->add_tag(ReblogSourceLink => \&hdlr_reblog_source_url);
MT::Template::Context->add_tag(ReblogSourceURL => \&hdlr_reblog_source_url);

sub hdlr_reblog_source_url {
    my $ctx = shift;
    my $f = $ctx->stash('reblog_source');
    return $f->{url};
}

MT::Template::Context->add_tag(ReblogSourceXMLLink => \&hdlr_reblog_source_feed_url);
MT::Template::Context->add_tag(ReblogSourceFeedURL => \&hdlr_reblog_source_feed_url);

sub hdlr_reblog_source_feed_url {
    my $ctx = shift;
    my $f = $ctx->stash('reblog_source');
    return $f->{feed_url};
}

MT::Template::Context->add_tag(ReblogSourceTitle => \&hdlr_reblog_source);
MT::Template::Context->add_tag(ReblogSource => \&hdlr_reblog_source);

sub hdlr_reblog_source {
    my $ctx = shift;
    my $f = $ctx->stash('reblog_source');
    return $f->{title};
}


MT::Template::Context->add_tag(ReblogSourceMinDate => \&hdlr_reblog_source_min_date);
MT::Template::Context->add_tag(ReblogSourceMaxDate => \&hdlr_reblog_source_max_date);

sub hdlr_reblog_source_min_date {
    my $source = $_[0]->stash('reblog_source');

    my $att = $_[1];
    $att->{ts} = $source->{min_ts};

    return MT::Template::Context::_hdlr_date($_[0], $att);
}

sub hdlr_reblog_source_max_date {
    my $source = $_[0]->stash('reblog_source');

    my $att = $_[1];
    $att->{ts} = $source->{max_ts};

    return MT::Template::Context::_hdlr_date($_[0], $att);
}


MT::Template::Context->add_tag(ReblogSourceFreq => \&hdlr_reblog_source_freg);

sub hdlr_reblog_source_freg {
    my $ctx = shift;
    my $att = shift;

    my $f = $ctx->stash('reblog_source');
    my ($scale, $scale_min, $scale_max) = (0,0,0);
    if(exists($att->{format}) && $att->{format} eq 'percent') {
	$scale = 1;
	$scale_min = 0;
	$scale_max = 100;
    }
    elsif(exists($att->{scale_min}) && exists($att->{scale_max})) {
# assume you've entered numbers
	$scale_min = $att->{scale_min};
	$scale_max = $att->{scale_max};	
	$scale = 1;
    }
    my $res = $f->{n};
    if($scale) {
	$res = $f->{n} / $f->{nTotal};
	$res = $res * ($scale_max - $scale_min) + $scale_min;
	if(exists($att->{truncate}) && $att->{truncate}) {
	    $res = int($res);
	}
    }

    return $res;
};


############################## END TAG HANDLERS FOR REBLOG SOURCES #######################
