#!/usr/bin/perl -w

# $Id: reblog_action.cgi,v 1.2.2.1 2005/09/22 23:01:14 mfrumin Exp $
#
# This file is part of Reblog: http://reblog.org
# Distributed under the Gnu Public License, see LICENSE
#
# Copyright 2004 Michael Frumin, Michal Migurski
# This is the Movable Type Plugin code to implement the reBlog functionality into Movable Type
#
# Developed by Michael Frumin (mfrumin AT eyebeam DOT org)
# Initially for Eyebeam's reBlog (http://www.eyebeam.org/reblog/)
#
# NOTE: This only works if you are using MySQL for the database backend for Movable Type
  
use strict;
use File::Spec;

my($MT_DIR);
BEGIN {
    if (0 && $0 =~ m!(.*[/\\])!) {
        $MT_DIR = $1;
    } else {
	$MT_DIR = $ENV{SCRIPT_FILENAME};
    }

    # back up to directory just above 'plugins'
    my ($vol, $dir, $f) = File::Spec->splitpath($MT_DIR);
    my (@dirs) = File::Spec->splitdir($dir);
    while(pop(@dirs) ne 'plugins' && scalar(@dirs) > 0){};
    push(@dirs, "");
    $MT_DIR = File::Spec->join(@dirs);

    unshift @INC, $MT_DIR . 'lib';
    unshift @INC, $MT_DIR . 'extlib';
}

eval {
    require MT::App::CMS;
    require MT::Author;
    my $app = MT::App::CMS->new( Config => $MT_DIR . 'mt.cfg',
                                 Directory => $MT_DIR )
        or die MT::App::CMS->errstr;
    local $SIG{__WARN__} = sub { $app->trace($_[0]) };

    my $cgi = $app->{query};

    my ($auth) = $app->login();

    if(!(defined($auth) && $auth)) {
	print $cgi->redirect($app->path . "mt.cgi");
	exit;
    }

    if(!ref($auth)) {
	$auth = MT::Author->load($auth);
    }


    $app->send_http_header;

    use MT::Util qw( encode_html );

    my $action = $cgi->param("action");
    my $blog_id = $cgi->param("blog_id");
    my $back_url = $cgi->param("back") || $ENV{HTTP_REFERER} || "";

    require MT::App::CMS;
    my $header = MT::App::CMS::build_page ($app, "header.tmpl", { author_name => $auth->username, no_breadcrumbs => 1 }) || "";
    my $footer = MT::App::CMS::build_page ($app, "footer.tmpl", {}) || "";

    print $header;

    my $back = ($back_url ? qq{&lt;&lt;&lt;&nbsp;<a href="} . encode_html($back_url) . qq{">go back from whence you came</a>} : "");


    my ($status, $version) = MT::Plugin::Reblog::check_reblog_status();

    if($status) {
	status_action_err($status, $version, $action, $cgi);
    }
    else {
	if(defined($blog_id) && $blog_id) {
	    my $blog = MT::Blog->load($blog_id);

	    if($action eq 'config') {
		do_config($blog, $back_url, $cgi);
	    }
	    elsif($action eq  'import') {
		do_import($app, $auth, $blog, $cgi);
	    }
	    elsif($action eq  'cleanup') {
		do_cleanup($app, $auth, $blog, $cgi);
	    }
	}
	else {
	    print qq{We are sorry.  To use the Reblog plugin, you must be coming from a page specific to one blog.<BR>
			 Please go to the Main page "Edit Entries" page for your blog, and click on the Reblog plugin action links.
			 <BR>
		     };
	}
    }


    print qq{<BR>&lt;&lt;&lt;&nbsp;<a href="../mt.cgi">Back to Movable Type</a><BR>\n};
    print ($back ? "<BR>$back<BR>" : "");
    print $footer;

};
if ($@) {
    print STDERR "ERROR: $@\n";
    print "Content-Type: text/html\n\n";
    print "Got an error: <BR><PRE>" . encode_html($@) . "</PRE>";
}

sub do_config {
    my ($blog, $back_url, $cgi) = @_;

    my $blog_id = $blog->id;
    my $source_rss = MT::Plugin::Reblog::source_rss($blog->id);
    
    my $source_rss_in = $cgi->param("source_rss");
    if(defined($source_rss_in) && $source_rss_in) {
	$source_rss = MT::Plugin::Reblog::source_rss($blog->id, $source_rss_in);
    }
    $source_rss = encode_html($source_rss);

    print qq{
	<form action="reblog_action.cgi" method="POST">
	    <input type="hidden" name="action" value="config">
	    <input type="hidden" name="blog_id" value="$blog_id">
	    <input type="hidden" name="back" value="$back_url">
			    Reblog Source Feed:<BR>
			    <input type="text" name="source_rss" size="100" value="$source_rss"><BR>
			    <input type="submit" name="submit" value="Submit"><BR>
			    </form>
			    };	    
}

sub do_import {
    my ($app, $auth, $blog, $cgi) = @_;

    my $source_rss = MT::Plugin::Reblog::source_rss($blog->id);

    print "Going to import entries from $source_rss<BR><BR>\n";
    
    my (@entries) = MT::Plugin::Reblog::import_entries($blog->id, $source_rss, $auth, 1);
    
    my $nBuilt = 0;
    foreach my $eRec (@entries) {
	
	print "Found Entry<BR>\n";
	print "Title: " . $eRec->{entry}->title . " (id = " . $eRec->{entry}->id . ")<BR>\n";
	print "Link: " . $eRec->{reblog_data}->link . "<BR>\n";
	print "Source: " . $eRec->{reblog_data}->source . "<BR>\n";
	print "Guid: " . $eRec->{reblog_data}->guid . "<BR>\n";
	print "Status: $eRec->{status}";
	
	if($eRec->{status} ne 'old') {
	    print ", (Re)building entry page.";
	    $app->rebuild_entry(Entry => $eRec->{entry}, Blog => $blog);
	    $nBuilt++;
	}
	print "<BR><BR>\n";
    }
    
    if($nBuilt > 0) {
	$app->rebuild_indexes(Blog => $blog);
	print "<BR>$nBuilt Entries rebuilt, rebuilding index pages as well<BR>\n";
    }
    
    my $blog_url = encode_html($blog->site_url);
    print qq{<BR>>>>&nbsp;<a href="$blog_url" target="$blog_url">See your blog at $blog_url</a><BR>\n};

    	    
}

sub do_cleanup {
    my ($app, $auth, $blog, $cgi) = @_;

    print MT::Plugin::Reblog::cleanup_reblog_data();
}

sub status_action_err {
    my ($status, $version, $action, $cgi) = @_;

    my $curr_version = $MT::Plugin::Reblog::VERSION;

    my $err = '';
    my $msg = '';

    if($status eq '-mysql') {
	$err .= qq{Sorry, but the reBlog plugin requires you to be running MT with a MySQL database};
    }
    elsif($status eq '-install') {
	$err .= qq{Sorry, you have not as of yet <i>installed</i> the reBlog plugin.<BR>Please <a href="reblog_install.cgi">click here to install the reBlog plugin</a>.};
    }
    elsif($status eq '-upgrade') {
	$err .= qq{Sorry, you have installed version $curr_version of the reBlog plugin but are still running $version and have not as of yet ran the upgrade script.<BR>Please <a href="reblog_upgrade.cgi?">click here to finalize your upgrade from v$version to v$curr_version</a>.};
    }

    if($err) {
	return '<div class="err">' . $err . '</div>';
    }
    return $msg;
}



