#!/usr/bin/perl
  
use strict;
use File::Spec;

# set this to the ID of the blog you want to import into
use constant RB_BLOG_ID => 1;

# set this to the MT login name of the user you want to import as
use constant RB_AUTHOR_NAME => 'reblogger';

my($MT_DIR);
BEGIN {
    $MT_DIR = $0;

    # back up to directory just above 'plugins'
    my ($vol, $dir, $f) = File::Spec->splitpath($MT_DIR);
    my (@dirs) = File::Spec->splitdir($dir);
    push(@dirs, "");
    $MT_DIR = File::Spec->join(@dirs);
    
    unshift @INC, $MT_DIR . 'lib';
    unshift @INC, $MT_DIR . 'extlib';

}

eval {
    require MT;
    require MT::Author;

    my $app = MT->new( Config => $MT_DIR . 'mt.cfg',
                                 Directory => $MT_DIR )
        or die MT->errstr;
    

    my $auth = MT::Author->load({ name => RB_AUTHOR_NAME });

    my $blog = MT::Blog->load(RB_BLOG_ID);

    do_import($app, $auth, $blog);

};
if ($@) {
    print STDERR "ERROR: $@\n";
    print "Got an error: " . $@ . "\n";
}

sub do_import {
    my ($app, $auth, $blog) = @_;

    my $source_rss = MT::Plugin::Reblog::source_rss($blog->id);

    print "Going to import entries from $source_rss\n";
    
    my (@entries) = MT::Plugin::Reblog::import_entries($blog->id, $source_rss, $auth, 1);
    
    my $nBuilt = 0;
    foreach my $eRec (@entries) {
	
	print "\nFound Entry\n";
	print "Title: " . $eRec->{entry}->title . " (id = " . $eRec->{entry}->id . ")\n";
	print "Link: " . $eRec->{reblog_data}->link . "\n";
	print "Source: " . $eRec->{reblog_data}->source . "\n";
	print "Guid: " . $eRec->{reblog_data}->guid . "\n";
	print "Status: $eRec->{status}";
	
	if($eRec->{status} ne 'old') {
	    print ", (Re)building entry page.";
	    $app->rebuild_entry(Entry => $eRec->{entry}, Blog => $blog);
	    $nBuilt++;
	}
	print "\n\n";
    }
    
    if($nBuilt > 0) {
	$app->rebuild_indexes(Blog => $blog);
	print "$nBuilt Entries rebuilt, rebuilding index pages as well\n";
    }
    	    
}
