<?php
/*----------------------------------------
$Id: Refeed-Plugin.php,v 1.1.2.2 2005/09/07 05:04:11 migurski Exp $
vim: ts=4 foldcolumn=4 foldmethod=marker

This file is part of Reblog: http://reblog.org
A derivative work of Feed On Feeds: http://feedonfeeds.com

Distributed under the Gnu Public License, see LICENSE

Copyright 2004 Michael Frumin, Michal Migurski
mike@stamen.com, http://stamen.com
mfrumin@eyebeam.org, http://eyebeam.org

Plugin Name: reBlog
Plugin URI: http://reblog.org/
Description: Accept input from reBlog, placing published items in regular WordPress weblog.
Version: 1.1
Author: Michal Migurski
Author URI: http://reblog.org/
----------------------------------------*/

    // Configuration {{{1
    
    // Complete URL to RSS 2.0 output from reBlog, typically ends with "out/rss.php?v=2".
    define('REBLOG_URL', 'http://localhost/~migurski/reblog/refeed/out/rss.php?v=2');

    // 1 if in debugging mode;
    // debugging means that file /tmp/reblog-fetch-effluvia.txt is written with
    // diagnostic information whenever this plug-in is run; see reblog_fetch()
    define('REBLOG_DEBUG', 0);

    // Where's magpie? Don't change this, in general.
    define('MAGPIE_DIR', dirname(__FILE__).'/magpierss/');
    
    // And what is our special reblog table called? Don't change this, in general.
    $GLOBALS['tablereblog'] = "{$GLOBALS['table_prefix']}posts_reblog";

    // Functions {{{1
    
   /** reblog_print_style {{{2
    * output CSS style declaration that placed reblog-form correctly
    * in the upper-right corner of WordPress admin screen.
    */
    function reblog_print_style()
    {
        print('
        <style type="text/css">
        #reblog-form
        {
            position: absolute;
            top: 5px;
            margin: 0;
            padding: 0;
            right: 3em;
            font-size: 20px;
            color: #666;
        }
        </style>
        ');
    }

   /** is_reblog_post {{{2
    * Check whether a given post has been reblogged or not, for use in templates.
    * $id is provided globally, through WP's template-like code.
    *
    * @return   mixed   false if the given post has not been reblogged,
    *                   text guid if it has.
    */
    function is_reblog_post()
    {
        $post = $GLOBALS['wpdb']->get_row(sprintf("SELECT r.guid AS guid
                                                       FROM `{$GLOBALS['tableposts']}` AS p
                                                       LEFT JOIN `{$GLOBALS['tablereblog']}` AS r
                                                       ON r.id = p.id
                                                       WHERE p.id = '%s'",
                                                   mysql_real_escape_string($GLOBALS['id'])));

        return $post->guid ? $post->guid : false;
    }
    
   /** reblog_is_installed {{{2
    * Check whether WP reBlog plug-in is installed by examining posts table.
    *
    * @return   bool    true iff plug-in is installed.
    */
    function reblog_is_installed()
    {
        return ($GLOBALS['wpdb']->get_row("SHOW TABLES LIKE '{$GLOBALS['tablereblog']}'")) ? true : false;
    }
    
   /** reblog_print_report {{{2
    * output HTML report, hope it goes in the upper-right corner as it should.
    * (see reblog_print_style())
    */
    function reblog_print_report()
    {
        echo $GLOBALS['REFEED_REPORT'];
    }
    
   /** reblog_act {{{2
    * perform whatever installation or fetching actions are needed.
    *
    * @return   string  HTML report, for placement in upper-right corner
    *                   of admin. (see reblog_print_report())
    */
    function reblog_act()
    {
        ob_start(); {
            print('<form action="#" method="post" id="reblog-form">');
        
            if(reblog_is_installed()) {
            
                if($_POST['reblog_action'] == 'Fetch reBlog') {
            
                    $fetched = reblog_fetch(REBLOG_URL);
                    
                    print(($fetched === false)
                        ? "<span style=\"color: red;\">Please use RSS 2.0 from reBlog.</span>\n"
                        : (is_array($fetched)
                            ? " (Added {$fetched[0]} items, Updated {$fetched[1]} items) "
                            : ' '));
        
                }
        
                print('<input type="submit" name="reblog_action" value="Fetch reBlog" />');
        
            } elseif($_POST['reblog_action'] == 'Install reBlog') {
        
                print(reblog_install()
                    ? "reBlog Plugin Installed.\n".'<input type="submit" name="reblog_action" value="Fetch reBlog" />'
                    : "<span style=\"color: red;\">reBlog Plugin Not Installed.</span>\n");
        
            } else {
                print('<input type="submit" name="reblog_action" value="Install reBlog" />');
            
            }
        
            print('</form>');
        } $report = ob_get_contents();
        ob_end_clean();
        
        $GLOBALS['REFEED_REPORT'] = $report;
    }
    
   /** reblog_install {{{2
    * Attempt to install reBlog plug-in by altering posts table.
    *
    * @return   bool    true iff plug-in is installed.
    *                   (see reblog_is_installed())
    */
    function reblog_install()
    {
        $GLOBALS['wpdb']->query("CREATE TABLE `{$GLOBALS['tablereblog']}` (
                                     id     INT(10) UNSIGNED NOT NULL,
                                     guid   VARCHAR(240) NOT NULL,
                                     
                                     UNIQUE (id),
                                     UNIQUE (guid)
                                 )");
    
        return reblog_is_installed();
    }
    
   /** reblog_fetch {{{2
    * Attempt to fetch posts from reBlog using included MagpieRSS library.
    *
    * @param    string  reblog_URL  complete URL to RSS 2.0 output from reBlog,
    *                               typically ends with "rss.php?v=1".
    *
    * @return   mixed   boolean false if fetch failed.
    *                   otherwise, array with number of new items fetched and
    *                   number of older items updated.
    */
    function reblog_fetch($reblog_URL)
    {
        // require the MagpieRSS library, for pulling in the code.
        if(file_exists(ABSPATH . WPINC . '/rss-functions.php')) {
            // hey, WP has one of its own
            require_once(ABSPATH . WPINC . '/rss-functions.php');

        } else {
            require_once(MAGPIE_DIR . 'rss_fetch.inc');
            require_once(MAGPIE_DIR . 'rss_utils.inc');

        }
        
        // magpie caching is teh lame
        $previous_MAGPIE_CACHE_ON = MAGPIE_CACHE_ON;
        define('MAGPIE_CACHE_ON', false);
        $rss = fetch_rss($reblog_URL);
        define('MAGPIE_CACHE_ON', $previous_MAGPIE_CACHE_ON);
        
        $added = $updated = 0;

        // ensure that the reblog feed is RSS 2.0
        if(($rss->feed_type != 'RSS') || ($rss->feed_version != '2.0')) {
            return false;
        
        } else {
        
            // perform most of the fetch inside an output buffer. facilitates the debugging logfile.
            ob_start(); {
            
                if(REBLOG_DEBUG) {
                    print_r($rss);
                    print_r($_POST);
                    print_r(debug_backtrace());
                }
            
                // iterate over each item in the fetched RSS
                foreach($rss->items as $i => $item) {
            
                    print("---- item {$i} ----\n");
                    print(sprintf("item[dc][date]: %s -> %s -> %s (%s)\n", $item['dc']['date'], parse_w3cdtf($item['dc']['date']), date('r', parse_w3cdtf($item['dc']['date'])), gmdate('r', parse_w3cdtf($item['dc']['date']))));

                    // like "Originally by so-and-so from elsewhere on some date"
                    $post_attribution = '<a href="'.htmlspecialchars($item['rb']['via_url']).'">Originally</a>';

                    if($item['rb']['source_author'])
                        $post_attribution .= ' by '.htmlspecialchars($item['rb']['source_author']);
                        
                    $post_attribution .= ' from <a href="'.htmlspecialchars($item['rb']['source_url']).'">'.htmlspecialchars($item['rb']['source']).'</a>';
                    
                    if($item['rb']['source_published_date'])
                        $post_attribution .= ' on '.date('F j, Y, g:ia', parse_w3cdtf($item['rb']['source_published_date']));
                        
                    // append attribution to post content
                    $post_content = "{$item['description']} <p><em>{$post_attribution}</em></p>";
                    
                    // determine whether the item's GUID has been encountered before
                    $post = $GLOBALS['wpdb']->get_row(sprintf("SELECT p.id AS id
                                                                   FROM `{$GLOBALS['tablereblog']}` AS r
                                                                   LEFT JOIN `{$GLOBALS['tableposts']}` AS p
                                                                   ON r.id = p.id
                                                                   WHERE r.guid = '%s'",
                                                               mysql_real_escape_string($item['rb']['guid'])));
                    
                    // update existing post if the GUIO was found, otherwise insert a new one.
                    if($post->id) {
                    
                        $q = sprintf("UPDATE `{$GLOBALS['tableposts']}`
                                         SET post_content = '%s',
                                             post_title = '%s',
                                             post_excerpt = '%s',
                                             post_name = '%s',
                                             post_modified = '%s',
                                             post_modified_gmt = '%s'
                                         WHERE id = '%s'",
                                     mysql_real_escape_string(format_to_post(balanceTags($post_content))),
                                     mysql_real_escape_string($item['title']),
                                     mysql_real_escape_string(format_to_post(balanceTags($item['summary']))),
                                     mysql_real_escape_string($item['title']),
                                     date('Y-m-d H:i:s'),
                                     gmdate('Y-m-d H:i:s'),
                                     mysql_real_escape_string($post->id));
                                     
                        $GLOBALS['wpdb']->query($q); print("{$q}\n");
                        
                        $updated++;
    
                    } else {
                    
                        $q = sprintf("INSERT INTO `{$GLOBALS['tableposts']}`
                                         (post_author, post_date, post_date_gmt, post_content, post_title, post_excerpt, post_name, post_modified, post_modified_gmt)
                                         VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s')",
                                     1, // pick an author, any author
                                     date('Y-m-d H:i:s', parse_w3cdtf($item['dc']['date'])),
                                     gmdate('Y-m-d H:i:s', parse_w3cdtf($item['dc']['date'])),
                                     mysql_real_escape_string(format_to_post(balanceTags($post_content))),
                                     mysql_real_escape_string($item['title']),
                                     mysql_real_escape_string(format_to_post(balanceTags($item['summary']))),
                                     mysql_real_escape_string($item['title']),
                                     date('Y-m-d H:i:s', parse_w3cdtf($item['dc']['date'])),
                                     gmdate('Y-m-d H:i:s', parse_w3cdtf($item['dc']['date'])));
                                     
                        $GLOBALS['wpdb']->query($q); //print("{$q}\n");
                        
                        $post = $GLOBALS['wpdb']->get_row("SELECT LAST_INSERT_ID() AS id FROM `{$GLOBALS['tableposts']}`");
        
                        $added++;
                        
                        $q = sprintf("INSERT IGNORE INTO `{$GLOBALS['tablereblog']}`
                                         (id, guid)
                                         VALUES ('%s', '%s')",
                                     mysql_real_escape_string($post->id),
                                     mysql_real_escape_string($item['rb']['guid']));
                                     
                        $GLOBALS['wpdb']->query($q); //print("{$q}\n");
                        
                        // pick a category, any category
                        $q = sprintf("INSERT IGNORE INTO `{$GLOBALS['tablepost2cat']}`
                                         (post_id, category_id)
                                         VALUES ('%s', '1')",
                                     mysql_real_escape_string($post->id));
                                     
                        $GLOBALS['wpdb']->query($q); //print("{$q}\n");
                    }
                        
                    unset($item['description']);
                    unset($item['summary']);
                    print_r($item);
                }
            
            } $debug_output = ob_get_contents();
            ob_end_clean();
            
            // write debugging logfile if needed
            if(REBLOG_DEBUG && ($fh = fopen('/tmp/reblog-fetch-effluvia.txt', 'w'))) {
                fwrite($fh, $debug_output);
                fclose($fh);
            }
        }
    
        return array($added, $updated);
    }
    
    // Do the work {{{1
    //$GLOBALS['REFEED_REPORT'] = reblog_act();
    add_action('admin_head', 'reblog_print_style');
    add_action('admin_head', 'reblog_act');
    add_action('admin_footer', 'reblog_print_report');

?>
